#!/usr/bin/env python3
import os
import csv
import datetime

regions = set()
region_stats = {}

alt_names = {
        "Czechia": ["Czech Republic"],
        "South Korea": ["Korea, South"],
        "United Kingdom": ["UK"],
        "Diamond Princess": ["Diamond Princess cruise ship"],
}

def translate_region(region):
    for orig, names in alt_names.items():
        for name in names:
            if region == name:
                return orig
    return region


fns = [fn for fn in os.listdir() if fn.endswith(".csv")]
fns.sort()
last = fns[-1]

month, day, year  = last.partition('.')[0].split("-")
last_d = datetime.date(int(year), int(month), int(day))

first_line = True
for line in csv.reader(open(last, "r")):
    if first_line:
        if "FIPS" in line[0]:
            fips = True
        else:
            fips = False
        first_line = False
        continue

    if fips :
        # skip US because the data for US is a big inconsistent mess
        if line[3] == 'US': continue
        if line[2]:
            region = line[2]
        else:
            region = line[3]
        confirmed = int(line[7])
        dead = int(float(line[8]))
    else:
        # skip US because the data for US is a big inconsistent mess
        if line[1] == 'US': continue
        if line[0]:
            region = line[0]
        else:
            region = line[1]
        confirmed = int(line[3])
        dead = int(float(line[4]))
    region = translate_region(region)
    if confirmed > 500:
        regions.add(region)

        region_stats[region] = {}
        region_stats[region]["last_day"] = last_d
        region_stats[region]["last_day_confirmed"] = confirmed
        region_stats[region]["last_day_dead"] = dead
        region_stats[region]["last_day_mortality"] = dead / confirmed

        if fips:
            if line[3] == "Mainland China" or line[3] == "China":
                region_stats[region]["region_extra"] = "China"
        else:
            if line[1] == "Mainland China" or line[1] == "China":
                region_stats[region]["region_extra"] = "China"

for fn in fns:
    f = open(fn)
    count = 0
    have = False
    first_line = True
    for line in csv.reader(f):
        if first_line:
            if "FIPS" in line[0]:
                fips = True
            else:
                fips = False
            first_line = False
            continue
        if fips :
            # skip US because the data for US is a big inconsistent mess
            if line[3] == 'US': continue
            if line[2]:
                region = line[2]
            else:
                region = line[3]
            if not line[7]:
                confirmed = 0
            else:
                confirmed = int(line[7])
            if not line[4]:
                dead = 0
            else:
                dead = int(line[8])
        else:
            # skip US because the data for US is a big inconsistent mess
            if line[1] == 'US': continue
            if line[0]:
                region = line[0]
            else:
                region = line[1]

            if not line[3]:
                confirmed = 0
            else:
                confirmed = int(line[3])
            if not line[4]:
                dead = 0
            else:
                dead = int(line[4])

        if region not in regions: continue

        month, day, year  = fn.partition('.')[0].split("-")
        d = datetime.date(int(year), int(month), int(day))

        if confirmed > 0 and "first_day" not in region_stats[region]:
            region_stats[region]['first_day'] = d
        if confirmed >= 100 and 'first_day_100' not in region_stats[region]:
            region_stats[region]['first_day_100'] = d
        if confirmed >= 200 and 'first_day_200' not in region_stats[region]:
            region_stats[region]['first_day_200'] = d
        if confirmed >= 500 and 'first_day_500' not in region_stats[region]:
            region_stats[region]['first_day_500'] = d
        if dead > 0 and "first_day_dead" not in region_stats[region]:
            region_stats[region]["first_day_dead"] = d

# for region, data in region_stats.items():
#     if region == "Diamon Pricess": continue
#     print(region)
#     print(f"Today days since 100 confirmed: {(data['last_day'] - data['first_day_100']).days}")
#     if 'first_day_dead' in data:
#         print(f"100 confirmed after X days: {(data['first_day_100']-data['first_day']).days}")
#         print(f"First dead after 100 confirmed: {(data['first_day_dead']-data['first_day_100']).days}")
#         print(f"First dead after 500 confirmed: {(data['first_day_dead']-data['first_day_500']).days}")
#     else:
#         print(f"Today days since 100 confirmed: {(data['last_day'] - data['first_day_100']).days}")
#         print(f"Today days since 500 confirmed: {(data['last_day'] - data['first_day_500']).days}")
#     print(f"Mortality (%): {data['last_day_mortality']*100:0.2}")
#     print()

f = open("index.html", "w")
f.write("""
<head>
<link rel="stylesheet" href="bootstrap.min.css">
</head>
<body>
<script src="jquery-3.4.1.slim.min.js"></script>
<script src="popper.min.js"></script>
<script src="bootstrap.min.js"></script>

<script src="jquery.dataTables.min.js"></script>
<script src="dataTables.bootstrap4.min.js"></script>
"""
)
f.write("""
<div style="padding-left: 30px; margin-top: 10px; margin-bottom: 20px; line-height: 2em;">
<code style="padding-bottom: 20px">Based on <a href="https://github.com/CSSEGISandData/COVID-19.git">https://github.com/CSSEGISandData/COVID-19.git</a> for regions with >=500 confirmed cases.</code><br>
<h2><code>* <span style="background-color: #ffc107;color: white;padding:5px">rows in orange</span> show regions without any reported mortality so far - for these regions we show number of days since 100/200/500 confirmed cases <b>till today</b> (not till first reported mortality).</code></h2>
</div>
""")
f.write("<table id='table' class='table table-dark table-striped' style='margin-bottom: 0px !important'>")
f.write("""
<thead class='thead-light'>
<tr>
<th>Region
<th style="color: red">First dead after 100 confirmed cases (days)
<th>First dead after 200 confirmed cases (days)
<th>First dead after 500 conifrmed cases (days)
<th>Mortality (%)
</tr>
</thead>
""")

for region, data in region_stats.items():
    if region == "Diamond Princess": continue
    if 'first_day_dead' in data:
        f.write("<tr>")
    else:
        f.write("<tr class='bg-warning'>")
    f.write("<td>")
    f.write(region)
    if "region_extra" in data:
        f.write(f" ({data['region_extra']})")

    if 'first_day_dead' in data:
        n = (data['first_day_dead']-data['first_day_100']).days
    else:
        n = (data['last_day']-data['first_day_100']).days

    f.write(f"<td data-order={n}>{n}{' *' if 'first_day_dead' not in data else ''}")

    if 'first_day_dead' in data:
        n = (data['first_day_dead']-data['first_day_200']).days
    else:
        n = (data['last_day']-data['first_day_200']).days

    f.write(f"<td data-order={n}>{n}{' *' if 'first_day_dead' not in data else ''}")

    if 'first_day_dead' in data:
        n = (data['first_day_dead']-data['first_day_500']).days
    else:
        n = (data['last_day']-data['first_day_500']).days

    f.write(f"<td data-order={n}>{n}{' *' if 'first_day_dead' not in data else ''}")

    f.write("<td>")
    f.write(f"{data['last_day_mortality']*100:.2f}")
    f.write("</tr>")
f.write("</table>")
f.write("""
<script>
$(document).ready(function() {
    $('#table').DataTable(
        {
            "searching": false,
            "paging": false,
            "info": false,
            "order": [[ 1, "desc" ]],
        }
    );

    // datatable adds some bullshit that breaks the layout
    // and adds white bars around the table or something
    // let's fix that :D
    $("#table_wrapper").replaceWith($("#table"));
} );
</script>
""")
